@@ -191,12 +191,11 @@ wordList = ['abandon', 'ability', 'able', 'about', 'above', 'absent', 'absorb',
             'wrist', 'write', 'wrong', 'yard', 'year', 'yellow', 'you', 'young', 'youth', 'zebra', 'zero', 'zone',
             'zoo']
 
-
 class Address(object):
     def __init__(self, address='', publicKey='', privateKey='', seed='', alias='', nonce=0, pywaves=pywaves):
         self.pywaves = pywaves
-        self.txSigner = TxSigner()
-        self.txGenerator = TxGenerator()
+        self.txSigner = TxSigner(pywaves)
+        self.txGenerator = TxGenerator(pywaves)
         if nonce < 0 or nonce > 4294967295:
             raise ValueError('Nonce must be between 0 and 4294967295')
         if seed:
@@ -622,7 +621,6 @@ class Address(object):
     def buy(self, assetPair, amount, price, maxLifetime=30 * 86400, matcherFee=pywaves.DEFAULT_MATCHER_FEE,
             timestamp=0, matcherFeeAssetId=''):
         assetPair.refresh()
-        #normPrice = int(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price)
         normPrice = int(round(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price))
         id = self._postOrder(assetPair.asset1, assetPair.asset2, b'\0', amount, normPrice, maxLifetime, matcherFee, timestamp, matcherFeeAssetId)
         if self.pywaves.OFFLINE:
@@ -633,7 +631,6 @@ class Address(object):
     def sell(self, assetPair, amount, price, maxLifetime=30 * 86400, matcherFee=pywaves.DEFAULT_MATCHER_FEE,
              timestamp=0, matcherFeeAssetId=''):
         assetPair.refresh()
-        #normPrice = int(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price)
         normPrice = int(round(pow(10, 8 + assetPair.asset2.decimals - assetPair.asset1.decimals) * price))
         id = self._postOrder(assetPair.asset1, assetPair.asset2, b'\1', amount, normPrice, maxLifetime, matcherFee, timestamp, matcherFeeAssetId)
         if self.pywaves.OFFLINE:
